from collections.abc import Sequence
from typing import Optional, Union

import atoti as tt
from atoti._docs_utils import STANDARD_EXTERNAL_TABLE_OPTIONS_KWARGS
from atoti.directquery._clustering_columns_parameter import CLUSTERING_COLUMNS_PARAMETER
from atoti.directquery._external_table_options import ExternalTableOptions
from atoti_core import doc

from .table import SnowflakeTable


class SnowflakeTableOptions(ExternalTableOptions[SnowflakeTable]):
    @doc(**STANDARD_EXTERNAL_TABLE_OPTIONS_KWARGS)
    def __init__(
        self,
        *,
        array_conversion: Optional[
            Union[tt.MultiColumnArrayConversion, tt.MultiRowArrayConversion]
        ] = None,
        clustering_columns: Sequence[str] = (),
        keys: Optional[Sequence[str]] = None,
    ) -> None:
        """Additional options about the external table to create.

        Args:
            {array_conversion}
            {clustering_columns}
            {keys}

        Example:
            .. doctest:: snowflake_table_options
                :hide:

                >>> import os
                >>> from atoti_directquery_snowflake import SnowflakeConnectionInfo
                >>> connection_info = SnowflakeConnectionInfo(
                ...     "jdbc:snowflake://"
                ...     + os.environ["SNOWFLAKE_ACCOUNT_IDENTIFIER"]
                ...     + ".snowflakecomputing.com/?user="
                ...     + os.environ["SNOWFLAKE_USERNAME"]
                ...     + "&database=TEST_RESOURCES"
                ...     + "&schema=TESTS",
                ...     password=os.environ["SNOWFLAKE_PASSWORD"],
                ... )
                >>> external_database = session.connect_to_external_database(
                ...     connection_info
                ... )

            .. doctest:: snowflake_table_options

                >>> from atoti_directquery_snowflake import (
                ...     SnowflakeTableOptions,
                ... )
                >>> external_table = external_database.tables["TUTORIAL", "SALES"]
                >>> table = session.add_external_table(
                ...     external_table,
                ...     table_name="Sales",
                ...     options=SnowflakeTableOptions(keys=["SALE_ID"]),
                ... )

            Transform the table to get arrays from values stored on multiple rows:

            .. doctest:: snowflake_table_options

                >>> multi_row_external_table = external_database.tables[
                ...     "TUTORIAL", "MULTI_ROW_QUANTITY"
                ... ]
                >>> multi_row_external_table.columns
                ['PRODUCT', 'INDEX', 'QUANTITY']
                >>> multi_row_table = session.add_external_table(
                ...     multi_row_external_table,
                ...     table_name="Sales (Multi row array)",
                ...     options=SnowflakeTableOptions(
                ...         array_conversion=tt.MultiRowArrayConversion(
                ...             index_column="INDEX",
                ...             array_columns=["QUANTITY"],
                ...         ),
                ...     ),
                ... )
                >>> multi_row_table.head()
                                                 QUANTITY
                PRODUCT
                product_2  [50.0, 65.0, 55.0, 30.0, 80.0]
                product_1  [10.0, 20.0, 15.0, 25.0, 10.0]

            Transform the table to get arrays from values stored on multiple columns.
            Here, the prefix ``"QUANTITY"`` will match the columns ``"QUANTITY_0"``...``"QUANTITY_4"`` and make it into a 5-element array named ``"QUANTITY"``.

            .. doctest:: snowflake_table_options

                >>> multi_column_external_table = external_database.tables[
                ...     "TUTORIAL", "MULTI_COLUMN_QUANTITY"
                ... ]
                >>> multi_column_external_table.columns
                ['PRODUCT', 'QUANTITY_0', 'QUANTITY_1', 'QUANTITY_2', 'QUANTITY_3', 'QUANTITY_4']
                >>> multi_column_table = session.add_external_table(
                ...     multi_column_external_table,
                ...     table_name="Sales (Multi column array)",
                ...     options=SnowflakeTableOptions(
                ...         keys=["PRODUCT"],
                ...         array_conversion=tt.MultiColumnArrayConversion(
                ...             column_prefixes=["QUANTITY"],
                ...         ),
                ...     ),
                ... )
                >>> multi_column_table.head()
                                                 QUANTITY
                PRODUCT
                product_1  [10.0, 20.0, 15.0, 25.0, 10.0]
                product_2  [50.0, 65.0, 55.0, 30.0, 80.0]

        """
        super().__init__(
            array_conversion=array_conversion,
            keys=keys,
            options={
                CLUSTERING_COLUMNS_PARAMETER: clustering_columns
                if clustering_columns
                else None
            },
        )
